@@ -9,12 +9,13 @@ from __future__ import annotations
 import codecs
 import string
 from abc import ABC, abstractmethod
-from typing import Any, Iterable, Iterator, Sequence
+from typing import Any, overload
+from collections.abc import Iterable, Iterator, Sequence
 
 from .pq import Escaping
 from .abc import AdaptContext
 from ._enums import PyFormat
-from ._compat import LiteralString
+from ._compat import LiteralString, Template
 from ._encodings import conn_encoding
 from ._transformer import Transformer
 
@@ -288,14 +289,25 @@ class SQL(Composable):
 
         return Composed(rv)
 
-    def join(self, seq: Iterable[Any]) -> Composed:
+    @overload
+    def join(self, seq: Iterable[Template]) -> Template: ...
+
+    @overload
+    def join(self, seq: Iterable[Any]) -> Composed: ...
+
+    def join(self, seq: Iterable[Any]) -> Composed | Template:
         """
         Join a sequence of `Composable`.
 
-        :param seq: the elements to join. Elements that are not `Composable`
-            will be considered `Literal`.
+        :param seq: the elements to join.
 
         Use the `!SQL` object's string to separate the elements in `!seq`.
+        Elements that are not `Composable` will be considered `Literal`.
+
+        If the arguments are `Template` instance, return a `Template` joining
+        all the items. Note that arguments must either be all templates or
+        none should be.
+
         Note that `Composed` objects are iterable too, so they can be used as
         argument for this method.
 
@@ -306,18 +318,30 @@ class SQL(Composable):
             >>> print(snip.as_string(conn))
             "foo", "bar", "baz"
         """
-        rv = []
+
         it = iter(seq)
         try:
-            rv.append(next(it))
+            first = next(it)
         except StopIteration:
-            pass
-        else:
-            for i in it:
-                rv.append(self)
-                rv.append(i)
+            return Composed([])
 
-        return Composed(rv)
+        if isinstance(first, Template):
+            items = list(first)
+            for t in it:
+                if not isinstance(t, Template):
+                    raise TypeError(f"can't mix Template and {type(t).__name__}")
+                items.append(self._obj)
+                items.extend(t)
+            return Template(*items)
+
+        cs = [first]
+        for i in it:
+            if isinstance(i, Template):
+                raise TypeError(f"can't mix Template and {type(i).__name__}")
+            cs.append(self)
+            cs.append(i)
+
+        return Composed(cs)
 
 
 class Identifier(Composable):
@@ -479,3 +503,51 @@ class Placeholder(Composable):
 # Literals
 NULL = SQL("NULL")
 DEFAULT = SQL("DEFAULT")
+
+
+def as_string(obj: Any, context: AdaptContext | None = None) -> str:
+    """Convert an object to a string according to SQL rules.
+
+    :param obj: the object to convert
+    :param context: the context in which to convert the object
+    :type context: `~psycopg.abc.AdaptContext` | `!None`
+
+    Adaptation happens according to the type of `!obj`:
+
+    - `Composable` objects are converted according to their
+      `~Composable.as_string()` method;
+    - `~string.templatelib.Template` strings are converted according to the
+      rules documented in :ref:`template-strings`;
+    - every other object is converted as it was :ref:`a parameter passed to a
+      query <types-adaptation>`.
+
+    If `!context` is specified then it is be used to customize the conversion.
+    for example using the encoding of a connection or the dumpers registered.
+    """
+    if isinstance(obj, Composable):
+        return obj.as_string(context=context)
+    elif isinstance(obj, Template):
+        from ._tstrings import as_string
+
+        return as_string(obj, context)
+    else:
+        return Literal(obj).as_string(context=context)
+
+
+def as_bytes(obj: Any, context: AdaptContext | None = None) -> bytes:
+    """Convert an object to a bytes string according to SQL rules.
+
+    :param obj: the object to convert
+    :param context: the context in which to convert the object
+    :type context: `~psycopg.abc.AdaptContext` | `!None`
+
+    See `as_string()` for details.
+    """
+    if isinstance(obj, Composable):
+        return obj.as_bytes(context=context)
+    elif isinstance(obj, Template):
+        from ._tstrings import as_bytes
+
+        return as_bytes(obj, context)
+    else:
+        return Literal(obj).as_bytes(context=context)
